@@ -7,11 +7,14 @@
 #include <QGroupBox>
 #include <QVBoxLayout>
 #include <QLabel>
+#include <QNetworkCookie>
 
 #include "gclive.h"
 #include "gcparser.h"
 #include "gchtmlparser.h"
 
+#define PLUGIN_NAME "GcLive"
+
 // The GcUrl class automatically creates urls for geocaching.com
 class GcUrl : public QUrl {
 public:
@@ -23,13 +26,13 @@ public:
 class GcNetworkRequest : public QNetworkRequest {
 public:
   GcNetworkRequest() : QNetworkRequest() {
-    setRawHeader("User-Agent", "CacheMe GcLive Plugin");
+    setRawHeader("User-Agent", "CacheMe " PLUGIN_NAME " Plugin");
   }
 };
 
 // return a string containing everything, the string str contains 
 // between the first occurances of the strings start and end
-QString GcLive::subString(QString &str, const QString &start, const QString &end) {
+QString GcLive::subString(const QString &str, const QString &start, const QString &end) {
 		
   int is = str.indexOf(start) + start.length();
   int ie = str.indexOf(end, is);
@@ -40,15 +43,12 @@ QString GcLive::subString(QString &str, const QString &start, const QString &end
   return str.mid(is, ie-is);  
 }
 
-void GcLive::handleViewStateField(const QByteArray &data) {
-  QString str(data);
+void GcLive::handleViewStateField(const QString &str) {
   QString lvCount = subString(str,  "__VIEWSTATEFIELDCOUNT\" value=\"", "\" />");
   int count = lvCount.isNull()?1:lvCount.toInt();
 
   QString lvResult;
 
-  Q_ASSERT(count == 1);  // make sure we really handle count != 1 correctly
-
   for (int i=0; i<count; i++) { 
     QString key = "__VIEWSTATE"+(!i?"":QString::number(i));
     
@@ -66,8 +66,7 @@ void GcLive::handleViewStateField(const QByteArray &data) {
 
 // ------------ handling of user token ---------------
 
-bool GcLive::decodeUserToken(const QByteArray &data) {
-  QString str(data);
+bool GcLive::decodeUserToken(const QString &str) {
   this->m_userToken = subString(str,  "userToken = '", "';");
   return !this->m_userToken.isNull();
 }
@@ -75,10 +74,11 @@ bool GcLive::decodeUserToken(const QByteArray &data) {
 void GcLive::requestUserToken() {
   GcNetworkRequest request;
   request.setUrl(GcUrl("/map/default.aspx"));
-  this->m_manager->get(request);
-  this->m_cacheProvider->emitBusy(true);
-
+  QNetworkReply *reply = this->m_manager->get(request);
+  emit notifyBusy(true);
   this->m_state = RequestedUserToken;
+
+  if(reply->error()) replyFinished(reply);
 }
 
 // --------------- log into gc.com --------------------
@@ -87,9 +87,11 @@ void GcLive::prepareLogin() {
   if(loginIsValid()) {
     GcNetworkRequest request;
     request.setUrl(GcUrl("/login/"));
-    this->m_manager->get(request);    
-    this->m_cacheProvider->emitBusy(true);
+    QNetworkReply *reply = this->m_manager->get(request);    
+    emit notifyBusy(true);
     this->m_state = SentLoginGET;
+
+    if(reply->error()) replyFinished(reply);
   } else {
     error(tr("No valid login specified, cache positions will be inaccurate!"));
     requestUserToken();
@@ -110,36 +112,47 @@ void GcLive::doLogin() {
     QUrl::toPercentEncoding(m_loginPassword);
  
   m_manager->post(request, data.toUtf8());
-  this->m_cacheProvider->emitBusy(true);
+
+  emit notifyBusy(true);
   this->m_state = SentLoginPOST;
 }
 
-GcLive::GcLive() : m_initialized(false), m_cacheList("GcLive"), 
-		   m_state(Idle), m_cacheProvider(NULL) {
+GcLive::GcLive() : m_initialized(false), m_cacheList(PLUGIN_NAME), 
+		   m_state(Idle), m_currentRequest(None) {
   qDebug() << __FUNCTION__;
 
   // try to get credentials from qsettings
   QSettings settings;
-  settings.beginGroup("GcLive");
+  settings.beginGroup(PLUGIN_NAME);
   m_loginName = settings.value("Name", "").toString();
   m_loginPassword = settings.value("Password", "").toString();
   settings.endGroup();
-}
-
-void GcLive::setCacheProvider(CacheProvider *cacheProvider) {
-  m_cacheProvider = cacheProvider;
 
   // setup network manager and listen for its replies
   this->m_manager = new QNetworkAccessManager(this);
 
+  // try to load stored cookies
+  QList<QNetworkCookie> cookies;
+  settings.beginGroup(PLUGIN_NAME);
+  settings.beginGroup("Cookies");
+  foreach(QString key, settings.allKeys()) 
+    cookies.append(QNetworkCookie(key.toAscii(), settings.value(key).toByteArray()));
+  settings.endGroup();
+  settings.endGroup();
+  this->m_manager->cookieJar()->setCookiesFromUrl(cookies, GcUrl(""));
+
   connect(this->m_manager, SIGNAL(finished(QNetworkReply*)),
 	  this, SLOT(replyFinished(QNetworkReply*)));
+}
 
+void GcLive::init() {
   prepareLogin();
 }
 
 GcLive::~GcLive() {
   qDebug() << __FUNCTION__;
+
+  delete this->m_manager;
 }
 
 QString GcLive::name() {
@@ -164,15 +177,12 @@ bool GcLive::busy() {
 }
 
 void GcLive::replyFinished(QNetworkReply *reply) {
-
-  // whatever happened, we are now idle again
-  if(reply->error() != QNetworkReply::NoError)
-    qDebug() << __FUNCTION__ << "Reply error:" << reply->errorString();
+  if(reply->error())
+    error(tr("Error") + ": " + reply->errorString());
 
   // invoke appropriate decoder
   if(reply->isFinished()) {
-
-    QByteArray allData = reply->readAll();
+    QString allData = QString::fromUtf8(reply->readAll());
 
     handleViewStateField(allData);
 
@@ -185,7 +195,7 @@ void GcLive::replyFinished(QNetworkReply *reply) {
       // if m_initialized is already true, then this is a 
       // (re-)login attempt due to user interaction
       if(m_initialized)
-	m_cacheProvider->emitSettingsChanged();
+	emit settingsChanged();
 
       // now everything is initialized
       m_initialized = true;
@@ -195,14 +205,32 @@ void GcLive::replyFinished(QNetworkReply *reply) {
       // check if we are successfully logged in
       if (allData.contains("You are logged in as")) {
 	qDebug() << __FUNCTION__ << "login successful";
+
+	QSettings settings;
+	settings.beginGroup(PLUGIN_NAME);
+	settings.beginGroup("Cookies");
+	QList<QNetworkCookie> cookies = 
+	  this->m_manager->cookieJar()->cookiesForUrl(GcUrl(""));
+
+	// save all non-session cookies
+	foreach(QNetworkCookie cookie, cookies) 
+	  if(!cookie.isSessionCookie()) 
+	    settings.setValue(cookie.name(), cookie.value());
+
+	settings.endGroup();
+	settings.endGroup();
+
 	requestUserToken();
       } else {
 	if(this->m_state == SentLoginGET) {
+	  qDebug() << __FUNCTION__ << "we are not already logged in, doing so ...";
 	  doLogin();
 	} else if(this->m_state == SentLoginPOST) {
 	  qDebug() << __FUNCTION__ << "login failed";
-	  error("");  // clear error to make sure more messages are displayed
-	  error(tr("Login failed!"));
+	  if(!reply->error()) {
+	    error("");  // clear error to make sure more messages are displayed
+	    error(tr("Login failed!"));
+	  }
 	  requestUserToken();
 	}
       }
@@ -211,50 +239,52 @@ void GcLive::replyFinished(QNetworkReply *reply) {
       
       // initialization is finished, any reply received now should
       // be a reply to a real request
-      
-      // save type of request that just finished
-      CacheProvider::RequestEntry::RequestType type = m_cacheProvider->type();
-      m_cacheProvider->done();
+      CurrentRequest type = this->m_currentRequest;
+      this->m_currentRequest = None;
 
-      GcParser gcParser;
-      GcHtmlParser gcHtmlParser;
+      emit done();
 
-      switch(type) {
-      case CacheProvider::RequestEntry::Overview: 
-	// start parser (and tell him whether we tried to load a "big" area)
-	if(gcParser.decodeOverview(allData, m_cacheList, m_bigArea))
-	  m_cacheProvider->emitReplyOverview(m_cacheList);
-	else
-	  error(gcParser.error());
-	break;
+      if(!reply->error()) {
+	GcParser gcParser;
+	GcHtmlParser gcHtmlParser;
 	
-      case CacheProvider::RequestEntry::Info: 
-	if(gcParser.decodeInfo(allData, m_cache))
-	  m_cacheProvider->emitReplyInfo(m_cache);
-	else
-	  error(gcParser.error());
-	break;
-	
-      case CacheProvider::RequestEntry::Detail: 
-	if(gcHtmlParser.decode(allData, m_cache)) 
-	  m_cacheProvider->emitReplyDetail(m_cache);
-	else
-	  error(gcHtmlParser.error());
-	break;
-	
-      default:
-	qDebug() << __FUNCTION__ << "unknown request type" << type;
-	Q_ASSERT(0);
-	break;
+	switch(type) {
+	case Overview: 
+	  // start parser (and tell him whether we tried to load a "big" area)
+	  if(gcParser.decodeOverview(allData, m_cacheList, m_bigArea))
+	    emit replyOverview(m_cacheList);
+	  else
+	    error(gcParser.error());
+	  break;
+	  
+	case Info: 
+	  if(gcParser.decodeInfo(allData, m_cache))
+	    emit replyInfo(m_cache);
+	  else
+	    error(gcParser.error());
+	  break;
+	  
+	case Detail: 
+	  if(gcHtmlParser.decode(allData, m_cache)) 
+	    emit replyDetail(m_cache);
+	  else
+	    error(gcHtmlParser.error());
+	  break;
+	  
+	default:
+	  qDebug() << __FUNCTION__ << "unknown request type" << type;
+	  Q_ASSERT(0);
+	  break;
+	}
       }
     }
   }
 
   qDebug() << __FUNCTION__ << "next, busy = " << busy();
   if(!busy()) 
-    m_cacheProvider->next();
+    emit next();
 
-  m_cacheProvider->emitBusy(false);
+  emit notifyBusy(false);
 }
 
 void GcLive::postJson(const QString &post) {
@@ -265,11 +295,14 @@ void GcLive::postJson(const QString &post) {
   request.setRawHeader("Content-Type", "application/json; charset=utf-8");
 
   m_manager->post(request, post.toUtf8());
-  this->m_cacheProvider->emitBusy(true);
+  emit notifyBusy(true);
 }
 
 void GcLive::processRequestOverview(const QGeoBoundingBox &area) {
   qDebug() << __PRETTY_FUNCTION__;
+
+  Q_ASSERT(this->m_currentRequest == None);
+  this->m_currentRequest = Overview;
   
   // check if bounding box doesn't exceed limits (40km diagonally)
   this->m_bigArea = area.topLeft().distanceTo(area.bottomRight()) > 40000;
@@ -297,8 +330,8 @@ int GcLive::getCacheId(const QString &name) {
 
   if(id < 0) {
     // remove current request and continue with the next one
-    m_cacheProvider->done();
-    m_cacheProvider->next();
+    emit done();
+    emit next();
 
     error(tr("Unable to determine cache id!"));
   }
@@ -309,6 +342,9 @@ int GcLive::getCacheId(const QString &name) {
 void GcLive::processRequestInfo(const QString &name) {
   qDebug() << __PRETTY_FUNCTION__ << name;
 
+  Q_ASSERT(this->m_currentRequest == None);
+  this->m_currentRequest = Info;
+  
   int id = getCacheId(name);
   if(id < 0) return;
 
@@ -328,26 +364,31 @@ void GcLive::processRequestDetail(const QString &name) {
     error(tr("Unable to determine cache guid!"));
 
     // just eat the request for now
-    m_cacheProvider->done();
-    m_cacheProvider->next();
+    emit done();
+    emit next();
 
     return;
   }
 
   qDebug() << __FUNCTION__ << m_cache.guid();
 
+  Q_ASSERT(this->m_currentRequest == None);
+  this->m_currentRequest = Detail;
+
   // now request:
   GcNetworkRequest request;
   request.setUrl(GcUrl("/seek/cdpf.aspx?guid=" + m_cache.guid() + "&lc=5"));
-  this->m_manager->get(request);
-  this->m_cacheProvider->emitBusy(true);
+  QNetworkReply *reply = this->m_manager->get(request);
+  emit notifyBusy(true);
+
+  if(reply->error()) replyFinished(reply);
 }
 
 void GcLive::applyChanges() {
   qDebug() << __PRETTY_FUNCTION__;
 
   QSettings settings;
-  settings.beginGroup("GcLive");
+  settings.beginGroup(PLUGIN_NAME);
   settings.setValue("Name", m_name->text());
   settings.setValue("Password", m_password->text());
   settings.endGroup();
@@ -361,7 +402,7 @@ void GcLive::createConfig(QDialog *parent, QVBoxLayout *box) {
   QVBoxLayout *vbox = new QVBoxLayout;
 
   QSettings settings;
-  settings.beginGroup("GcLive");
+  settings.beginGroup(PLUGIN_NAME);
 
   /* -------------- user name entry --------------- */
   vbox->addWidget(new QLabel(tr("Username") + ":"));
@@ -384,7 +425,14 @@ void GcLive::createConfig(QDialog *parent, QVBoxLayout *box) {
 
 // send an error message to the cache provider
 void GcLive::error(const QString &msg) {
-  m_cacheProvider->emitReplyError(tr("GcLive") + ": " + msg);
+  if(!msg.isEmpty())
+    emit replyError(tr(PLUGIN_NAME) + ": " + msg);
+  else
+    emit replyError("");
+}
+
+QObject *GcLive::getObject() {
+  return this;
 }
 
 Q_EXPORT_PLUGIN2(gclive, GcLive);
