@@ -175,13 +175,13 @@ CacheProvider::CacheProvider() : m_currentPlugin(NULL) {
       pluginsDir.dirName().toLower() == "release")
     pluginsDir.cdUp();
 #endif
+
   loadPluginsInDir(pluginsDir);
 
 #ifdef LIBDIR
   if(m_cacheProviderPluginList.isEmpty()) {
     // if no local plugins were found -> try the intallation path
     pluginsDir = QDir(QDir::toNativeSeparators(LIBDIR));  
-    pluginsDir.cd(APPNAME);
     loadPluginsInDir(pluginsDir);
   }
 #endif
@@ -195,6 +195,12 @@ CacheProvider::CacheProvider() : m_currentPlugin(NULL) {
 
 CacheProvider::~CacheProvider() {
   qDebug() << __FUNCTION__;
+
+#if 0
+  foreach(CacheProviderPlugin *plugin, m_cacheProviderPluginList) 
+    delete plugin;
+#endif
+
   delete this->m_pending;
 }
 
@@ -254,8 +260,8 @@ void CacheProvider::emitSettingsChanged() {
   emit settingsChanged();
 }
 
-void CacheProvider::emitBusy(bool on) {
-  emit busy(on);
+void CacheProvider::emitNotifyBusy(bool on) {
+  emit notifyBusy(on);
 }
 
 QString CacheProvider::name() {
@@ -284,8 +290,37 @@ void CacheProvider::processRequestDetail(const QString &id) {
 }
 
 void CacheProvider::start() {
-  foreach(CacheProviderPlugin *plugin, m_cacheProviderPluginList) 
-    plugin->setCacheProvider(this);
+  foreach(CacheProviderPlugin *cplugin, m_cacheProviderPluginList) {
+    //    QObject *plugin = dynamic_cast<QObject*>(cplugin);
+    QObject *plugin = cplugin->getObject();
+    qDebug() << __FUNCTION__ << cplugin << plugin;
+
+    // connect to plugin
+    connect(plugin, SIGNAL(replyOverview(const CacheList &)), 
+	    this, SLOT(emitReplyOverview(const CacheList &)));
+
+    connect(plugin, SIGNAL(replyInfo(const Cache &)), 
+	    this, SLOT(emitReplyInfo(const Cache &)));
+
+    connect(plugin, SIGNAL(replyDetail(const Cache &)), 
+	    this, SLOT(emitReplyDetail(const Cache &)));
+
+    connect(plugin, SIGNAL(replyError(const QString &)), 
+	    this, SLOT(emitReplyError(const QString &)));
+    
+    connect(plugin, SIGNAL(settingsChanged()), 
+	    this, SLOT(emitSettingsChanged()));
+    
+    connect(plugin, SIGNAL(notifyBusy(bool)), 
+	    this, SLOT(emitNotifyBusy(bool)));
+    
+    connect(plugin, SIGNAL(done()), this, SLOT(done()));
+    
+    connect(plugin, SIGNAL(next()), this, SLOT(next()));
+
+    // call plugins init function
+    cplugin->init();
+  }
 }
 
 void CacheProvider::pluginConfig(QDialog *parent, QVBoxLayout *box) {
