@@ -29,21 +29,27 @@ HtmlView::HtmlView(const Description &desc, QWidget *parent) : QTextBrowser(pare
 }
  
 QVariant HtmlView::loadResource ( int, const QUrl &url) {
- 
-  if(!m_images.contains(url)) {
-    qDebug() << __FUNCTION__ << url;
+  QUrl localUrl(url); 
+
+  if(localUrl.host().isNull()) {
+    localUrl.setScheme("http");
+    localUrl.setHost("www.geocaching.com");
+  }
+  
+  if(!m_images.contains(localUrl)) {
+    qDebug() << __FUNCTION__ << localUrl;
 
     // request download of image
     QNetworkRequest request;
-    request.setUrl(url);
+    request.setUrl(localUrl);
     this->m_manager->get(request);
     
-    m_images.insert(url, QPixmap());
+    m_images.insert(localUrl, QPixmap());
 
     return QVariant();
   }
 
-  return m_images.value(url);
+  return m_images.value(localUrl);
 }
 
 void HtmlView::replyFinished(QNetworkReply *reply) {
