@@ -2,6 +2,7 @@
 #include <QTranslator>
 #include <QDir>
 #include <QLocale>
+#include <QDebug>
 
 #include "mainwindow.h"
 #include "locationprovider.h"
@@ -16,7 +17,6 @@ int main(int argc, char **argv) {
   // make sure the geoservice plugins in /usr/lib/cacheme/plugins
   // can be found
   app.addLibraryPath(QDir::toNativeSeparators(LIBDIR) + 
-		     QDir::separator() + APPNAME +
 		     QDir::separator() + "plugins");
 #endif  
 
@@ -25,7 +25,6 @@ int main(int argc, char **argv) {
   if(!translator.load(QString(APPNAME) + "_" + locale))
     translator.load(QString(APPNAME) + "_" + locale, 
 		    QDir::toNativeSeparators(DATADIR) + 
-		    QDir::separator() + APPNAME +	
 		    QDir::separator() + "lang");
 #else
   translator.load(QString(APPNAME) + "_" + locale);
@@ -40,6 +39,7 @@ int main(int argc, char **argv) {
 
   LocationProvider locationProvider;
 
+  qDebug() << __FUNCTION__ << "creating main window!";
   MainWindow *mainWindow = new MainWindow(&locationProvider);
   mainWindow->show();
 
