@@ -13,37 +13,18 @@
 #include <QSettings>
 
 #ifndef Q_WS_MAEMO_5
-#include <QStatusBar>
-#include <QProgressBar>
 #include <QVBoxLayout>
 #else
 #include <QMaemo5InformationBox>
 #endif
 
 void MainWindow::createMenu() {
-  QMenu *menu;
-  QAction *action;
-
-#ifndef Q_WS_MAEMO_5
-  menu = new QMenu(tr("&File"), this);
-  action = new QAction(tr("E&xit"), this);
-  action->setShortcuts(QKeySequence::Quit);
-  connect(action, SIGNAL(triggered()), this, SLOT(close()));
-  menu->addAction(action);
-  menuBar()->addMenu(menu);
+#if !defined(Q_WS_MAEMO_5) && !defined(Q_OS_SYMBIAN)
+  addMenuEntry(tr("E&xit"), this, SLOT(close()));
 #endif
   
-  menu = new QMenu(tr("&Settings"), this);
-  action = new QAction(tr("&Settings"), this);
-  connect(action, SIGNAL(triggered()), this, SLOT(settingsdialog()));
-  menu->addAction(action);
-  menuBar()->addMenu(menu);
-
-  menu = new QMenu(tr("&Help"), this);
-  action = new QAction(tr("&About"), this);
-  connect(action, SIGNAL(triggered()), this, SLOT(about()));
-  menu->addAction(action);
-  menuBar()->addMenu(menu);
+  addMenuEntry(tr("&Settings"), this, SLOT(settingsdialog()));
+  addMenuEntry(tr("&About"), this, SLOT(about()));
 }
 
 void MainWindow::settingsdialog() {
@@ -115,7 +96,7 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   QObject::connect(this->m_cacheProvider, SIGNAL(settingsChanged()),
 		   this, SLOT(reloadCaches()));
 
-  QObject::connect(this->m_cacheProvider, SIGNAL(busy(bool)),
+  QObject::connect(this->m_cacheProvider, SIGNAL(notifyBusy(bool)),
 		   this, SLOT(setBusy(bool)));
 
   QObject::connect(this->m_mapWidget, SIGNAL(mapChanged()),
@@ -130,6 +111,9 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   QObject::connect(this, SIGNAL(zoomOutPressed()),
 		   this->m_mapWidget, SLOT(zoomOut()));
 
+  QObject::connect(this->m_mapWidget, SIGNAL(fullscreen()),
+		   this, SLOT(toggleFullscreen()));
+
   // connect to map to handle clicks on cache icons ...
   QObject::connect(this->m_mapWidget, SIGNAL(cacheClicked(const QString &)),
 		   this, SLOT(selectCache(const QString &)));
@@ -148,17 +132,13 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   scene->addItem(m_mapWidget);
   setCentralWidget(m_view);
 
-#ifndef Q_WS_MAEMO_5
-  // on maemo5 we use QMaemo5InformationBox
-  setStatusBar(new QStatusBar(this));
-#endif
-
   m_cacheProvider->start();
 }
 
 MainWindow::~MainWindow() {
   qDebug() << __FUNCTION__;
 
+  delete m_cacheProvider;
   delete m_serviceProvider;
 }
 
@@ -191,8 +171,7 @@ void MainWindow::statusMessage(const QString &message) {
 #ifdef Q_WS_MAEMO_5
   QMaemo5InformationBox::information(this, message);
 #else
-  statusBar()->setProperty("error", false);  
-  statusBar()->showMessage(message, 5000);
+  this->m_mapWidget->addBanner(message);
 #endif
 }
 
@@ -213,31 +192,14 @@ void MainWindow::showDetail(const Cache &cache) {
 void MainWindow::setBusy(bool on) {
   static int busyCount = 0;
 
+  if((!on && busyCount == 1) || ( on && busyCount <= 0)) {
 #ifdef Q_WS_MAEMO_5
-  if((!on && busyCount  > 0) || ( on && busyCount <= 0))
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, on);
 #else
-  static QProgressBar *progressBar = NULL;
-  if(on && busyCount <= 0) {
-    if(!progressBar) {
-      progressBar = new QProgressBar();
-      progressBar->setMinimum(0);
-      progressBar->setMaximum(0);
-      progressBar->setValue(25);
-      progressBar->setMaximumWidth(128);
-      progressBar->setMaximumHeight(14);  // seems to be useful
-      statusBar()->addPermanentWidget(progressBar);
-    } else {
-      statusBar()->addPermanentWidget(progressBar);
-      progressBar->reset();
-      progressBar->show();
-    }
-  } else if(!on && busyCount > 0) {
-    statusBar()->removeWidget(progressBar);
-  }
+    m_mapWidget->setBusy(on);
 #endif
+  }
 
   if(on)                        busyCount++;
   else if(!on && busyCount > 0) busyCount--;
 }
-
