@@ -4,17 +4,19 @@
 #include <QGraphicsSceneMouseEvent>
 #include <QGeoMapOverlay>
 #include <QSettings>
+#include <math.h>
 
 #include "mapwidget.h"
+#include "size.h"
 
-#if(QTM_VERSION < 0x010200)
-#error "QtMobility version 1.2 or higher required!"
+#if(QTM_VERSION < 0x010100)
+#error "QtMobility version 1.1 or higher required!"
 #endif
 
 void MapWidget::updateZoomButtons() {
   this->m_mapOverlay->enableZoomButtons(
-		zoomLevel() < maximumZoomLevel(),
-		zoomLevel() > minimumZoomLevel());
+	zoomLevel() < maximumZoomLevel(),
+	zoomLevel() > minimumZoomLevel());
   
   update();
 }
@@ -22,47 +24,48 @@ void MapWidget::updateZoomButtons() {
 MapWidget::MapWidget(QGeoMappingManager *mgr) : 
   QGraphicsGeoMap(mgr), m_bubble(NULL), m_posIsValid(false), m_gpsMarker(NULL),
   m_manager(mgr) {
-
+  
   // try to get default from qsettings, otherwise start at berlin's zoo
   QSettings settings;
   settings.beginGroup("Map");
   setCenter(QGeoCoordinate(
-           settings.value("Center/Latitude", 52.514).toFloat(),
-	   settings.value("Center/Longitude", 13.3611).toFloat()));
+		   settings.value("Center/Latitude", 52.514).toFloat(),
+		   settings.value("Center/Longitude", 13.3611).toFloat()));
   setZoomLevel(settings.value("ZoomLevel", 15).toInt());
   setMapType((QGraphicsGeoMap::MapType)settings.value("Type", 
-	      QGraphicsGeoMap::StreetMap).toInt());
-  m_iconSize = settings.value("IconSize", 48).toInt();  
+			      QGraphicsGeoMap::StreetMap).toInt());
   m_gpsLocked = settings.value("GpsLocked", false).toBool();
   m_precisionAlpha = settings.value("PrecisionAlpha", 32).toInt();
   settings.endGroup();
-  m_iconLoader = new IconLoader(m_iconSize);
-
+  m_iconLoader = new IconLoader(Size::MAP_WIDGET_ICON_SIZE);
+  
   // create the control overlays
   this->m_mapOverlay = new MapOverlay(this);
-
+  this->m_mapBanner = new MapBanner(this);
+  this->m_mapSpinner = NULL;
+  
   // and update it to reflect restored map state
   updateZoomButtons();
   if(m_gpsLocked)
     this->m_mapOverlay->changeGpsButton(true);
-
+  
   // create a group for the cache icons
   m_cacheGroup = new QGeoMapGroupObject();
   m_cacheGroup->setZValue(0);
   addMapObject(m_cacheGroup);
-
+  
   // create a gps marker and precision indicator, but hide them
   QPixmap *markerIcon = m_iconLoader->load("gps_marker");
   if(markerIcon) {
     m_gpsMarker = new QGeoMapPixmapObject(QGeoCoordinate(0,0), 
-		     QPoint(-m_iconLoader->size()/2, -m_iconLoader->size()/2),
+		  QPoint(-m_iconLoader->size()/2, -m_iconLoader->size()/2),
 		  *markerIcon);
 
     m_gpsMarker->setZValue(2);
     m_gpsMarker->setVisible(false);
     addMapObject(m_gpsMarker);
   }
-
+  
   m_gpsPrecision = new QGeoMapCircleObject();
   m_gpsPrecision->setPen(QPen((QColor(0, 0, 0, m_precisionAlpha))));
   m_gpsPrecision->setBrush(QBrush(QColor(0, 0, 0, m_precisionAlpha/2)));
@@ -74,7 +77,7 @@ MapWidget::MapWidget(QGeoMappingManager *mgr) :
   this->m_timer->setSingleShot(true);
   QObject::connect(this->m_timer, SIGNAL(timeout()),
 		   this, SLOT(reloadTimerExpired()));
-
+  
   // connect to map signals to be able to handle viewport changes
   QObject::connect(this, SIGNAL(centerChanged(const QGeoCoordinate &)),
 		   this, SLOT(centerChangedEvent(const QGeoCoordinate &)));
@@ -91,7 +94,6 @@ MapWidget::~MapWidget() {
   settings.setValue("Center/Latitude", center().latitude());
   settings.setValue("Center/Longitude", center().longitude());
   settings.setValue("ZoomLevel", zoomLevel());
-  settings.setValue("IconSize", m_iconSize);
   settings.setValue("GpsLocked", m_gpsLocked);  
   settings.setValue("PrecisionAlpha", m_precisionAlpha);
   settings.endGroup();
@@ -99,6 +101,10 @@ MapWidget::~MapWidget() {
   delete m_iconLoader;
   delete m_timer;
   delete m_mapOverlay;
+  delete m_mapBanner;
+
+  if(this->m_mapSpinner) 
+    delete m_mapSpinner;
 }
 
 // ------------ slots to handle overlay button clicks --------------
@@ -107,9 +113,10 @@ void MapWidget::zoomIn() {
     setZoomLevel(zoomLevel()+1);
     updateZoomButtons();
   } 
+}
 
-  if(zoomLevel() == maximumZoomLevel()) 
-    emit showMessage(tr("Maximum zoom level reached"));
+void MapWidget::addBanner(const QString &msg) {
+  this->m_mapBanner->message(msg);
 }
 
 void MapWidget::zoomOut() {
@@ -117,9 +124,6 @@ void MapWidget::zoomOut() {
     setZoomLevel(zoomLevel()-1);
     updateZoomButtons();
   }
-
-  if(zoomLevel() == minimumZoomLevel()) 
-    emit showMessage(tr("Minimum zoom level reached"));
 }
 
 void MapWidget::gpsFollow() {
@@ -128,6 +132,10 @@ void MapWidget::gpsFollow() {
   update();
 }
 
+void MapWidget::toggleFullscreen() {
+  emit fullscreen();
+}
+
 void MapWidget::resizeEvent(QGraphicsSceneResizeEvent *event) {
   QGraphicsGeoMap::resizeEvent(event);
   reload();
@@ -141,8 +149,22 @@ void MapWidget::zoomLevelChangedEvent(qreal) {
   reload();
 }
 
-void MapWidget::centerChangedEvent(const QGeoCoordinate &) {
-  reload();
+void MapWidget::centerChangedEvent(const QGeoCoordinate &newCenter) {
+
+  if(!m_lastUpdateCenter.isValid()) {
+    reload();
+    return;
+  }
+
+  QPointF lastScreenPos(coordinateToScreenPosition(m_lastUpdateCenter));
+  QPointF newScreenPos(coordinateToScreenPosition(newCenter));
+
+  // if either horizontal or vertical moved more than 10% of screen, then
+  // update
+  if(fabs(lastScreenPos.x() - newScreenPos.x()) > size().width()/10 ||
+     fabs(lastScreenPos.y() - newScreenPos.y()) > size().height()/10) {
+    reload();
+  }
 }
 
 void MapWidget::hideBubble() {
@@ -307,6 +329,8 @@ void MapWidget::reloadTimerExpired() {
 
   // check if viewport actually changed
   if(m_currentViewport != viewport()) {
+    m_lastUpdateCenter = center();
+
     emit mapChanged();
     m_currentViewport = viewport();
   }
@@ -361,4 +385,19 @@ void MapWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 
   // then draw our overlay on top
   this->m_mapOverlay->paint(painter);
+  this->m_mapBanner->paint(painter);
+
+  if(this->m_mapSpinner)
+    this->m_mapSpinner->paint(painter);
+}
+
+void MapWidget::setBusy(bool on) {
+  if(on) {
+    Q_ASSERT(!m_mapSpinner);
+    m_mapSpinner = new MapSpinner(this);
+  } else {
+    Q_ASSERT(m_mapSpinner);
+    delete m_mapSpinner;
+    m_mapSpinner = NULL;
+  }
 }
